@@ -1,7 +1,7 @@
 /*
  * guile.c - interface to Guile core library
  *
- * Copyright (C) 2011-2013 Thien-Thi Nguyen
+ * Copyright (C) 2011-2014, 2020, 2021 Thien-Thi Nguyen
  * Copyright (C) 2001, 2002, 2003 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2002 Andreas Rottmann <a.rottmann@gmx.at>
  * Copyright (C) 2001 Raimund Jacob <raimi@lkcc.org>
@@ -176,6 +176,16 @@ guile_get_current_load_port (void)
   return SCM_UNDEFINED;
 }
 
+#if GI_LEVEL_2_2                        /* TODO: Do it the gi.[hc] way.  */
+#define PORT_FILENAME(x)  scm_port_filename (x)
+#define PORT_LINUM(x)     gi_scm2int (scm_port_line (x))
+#define PORT_COL(x)       gi_scm2int (scm_port_column (x))
+#else
+#define PORT_FILENAME(x)  SCM_FILENAME (x)
+#define PORT_LINUM(x)     SCM_LINUM (x)
+#define PORT_COL(x)       SCM_COL (x)
+#endif
+
 /*
  * Report some error at the current scheme position.  Prints to stderr
  * but lets the program continue.  The format string @var{format} does not
@@ -192,7 +202,7 @@ guile_error (char *format, ...)
        Guile 1.8 but not 2.0 (which is succeptible to segv).
        TODO: Do all validation in ‘guile_get_current_load_port’.  */
     && SCM_OPINPORTP (lp);
-  SCM filename = lp_valid_p ? SCM_FILENAME (lp) : SCM_BOOL_F;
+  SCM filename = lp_valid_p ? PORT_FILENAME (lp) : SCM_BOOL_F;
   char file[1024];
 
   /* guile counts lines from 0, we have to add one */
@@ -200,8 +210,8 @@ guile_error (char *format, ...)
            (lp_valid_p && GI_GET_XREP_MAYBE (file, filename)
             ? file
             : "undefined"),
-           lp_valid_p ? (long) SCM_LINUM (lp) + 1 : 0,
-           lp_valid_p ? (long) SCM_COL (lp) : 0);
+           lp_valid_p ? (long) PORT_LINUM (lp) + 1 : 0,
+           lp_valid_p ? (long) PORT_COL (lp) : 0);
 
   va_start (args, format);
   vfprintf (stderr, format, args);
@@ -1163,7 +1173,7 @@ Emit error messages (to stderr).  */)
   /* Is that TCP?  */
   if (!strcmp (proto, PORTCFG_TCP))
     {
-      int port;
+      int port = 0;
       cfg->proto = SVZ_PROTO_TCP;
       err |= optionhash_extract_int (options, PORTCFG_PORT, 0, 0,
                                      &port, action);
@@ -1180,7 +1190,7 @@ Emit error messages (to stderr).  */)
   /* Maybe UDP?  */
   else if (!strcmp (proto, PORTCFG_UDP))
     {
-      int port;
+      int port = 0;
       cfg->proto = SVZ_PROTO_UDP;
       err |= optionhash_extract_int (options, PORTCFG_PORT,
                                      0, 0, &port, action);
