@@ -1,7 +1,7 @@
 /*
  * http-cgi.c - http cgi implementation
  *
- * Copyright (C) 2011-2013 Thien-Thi Nguyen
+ * Copyright (C) 2011-2014, 2020, 2021 Thien-Thi Nguyen
  * Copyright (C) 2000, 2001, 2003, 2004, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -256,7 +256,7 @@ http_cgi_read (svz_socket_t *sock)
 int
 http_cgi_write (svz_socket_t *sock)
 {
-  int do_write;
+  size_t do_write;
   int num_written;
   http_socket_t *http = sock->data;
 
