@@ -1,7 +1,7 @@
 /*
  * core.c - socket and file descriptor core implementations
  *
- * Copyright (C) 2011-2013 Thien-Thi Nguyen
+ * Copyright (C) 2011-2014, 2020, 2021 Thien-Thi Nguyen
  * Copyright (C) 2001, 2002, 2003 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -491,6 +491,16 @@ svz_sendfile (int out_fd, int in_fd, off_t *offset, size_t count)
   *offset += sbytes;
   ret = ret ? -1 : (int) sbytes;
 
+#elif defined (__APPLE__) && defined (__MACH__)
+
+  /* Mac OS X: Almost identical to FreeBSD except that the count parameter
+     is reused to capture sent bytes. */
+
+  off_t sbytes = count;
+  ret = sendfile (in_fd, out_fd, *offset, &sbytes, NULL, 0);
+  *offset += sbytes;
+  ret = ret ? -1 : (int) sbytes;
+
 #elif defined (__MINGW32__)
 
   /* ‘TransmitFile’
