@@ -1,7 +1,7 @@
 /*
  * server-socket.c - server sockets for TCP, UDP, ICMP and pipes
  *
- * Copyright (C) 2011-2013 Thien-Thi Nguyen
+ * Copyright (C) 2011-2014, 2020, 2021 Thien-Thi Nguyen
  * Copyright (C) 2000, 2001, 2002, 2003 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -52,6 +52,7 @@
 #include "libserveez/icmp-socket.h"
 #include "libserveez/server-core.h"
 #include "libserveez/server.h"
+#include "libserveez/binding.h"
 #include "libserveez/portcfg.h"
 #include "libserveez/server-socket.h"
 #include "misc-macros.h"
@@ -62,7 +63,7 @@
  * them by return a non-zero value.
  */
 static int
-svz_sock_idle_protect (svz_socket_t *sock)
+idle_protect (svz_socket_t *sock)
 {
   svz_portcfg_t *port = svz_sock_portcfg (sock);
 
@@ -79,7 +80,7 @@ svz_sock_idle_protect (svz_socket_t *sock)
 }
 
 /* This is 1 if ‘time_t’ cannot be cast into a ‘void *’.  If that's the
-   case, ‘svz_sock_check_frequency’ stores pointers to ‘svz_malloc’d
+   case, ‘check_frequency’ stores pointers to ‘svz_malloc’d
    values in the array (arranging to ‘svz_free’ them when done).  */
 #define TIME_T_TOO_FAT  (SIZEOF_VOID_P < sizeof (time_t))
 
@@ -90,7 +91,7 @@ svz_sock_idle_protect (svz_socket_t *sock)
  * otherwise non-zero.
  */
 static int
-svz_sock_check_frequency (svz_socket_t *parent, svz_socket_t *child)
+check_frequency (svz_socket_t *parent, svz_socket_t *child)
 {
   svz_portcfg_t *port = parent->port;
   char ip[64];
@@ -148,7 +149,7 @@ svz_sock_check_frequency (svz_socket_t *parent, svz_socket_t *child)
  * for @code{read_socket} for listening tcp sockets.
  */
 static int
-svz_tcp_accept (svz_socket_t *server_sock)
+tcp_accept (svz_socket_t *server_sock)
 {
   svz_t_socket client_socket;   /* socket to accept clients on */
   struct sockaddr_in client;    /* address of connecting clients */
@@ -208,9 +209,9 @@ svz_tcp_accept (svz_socket_t *server_sock)
   if ((sock = svz_sock_create (client_socket)) != NULL)
     {
       sock->flags |= SVZ_SOFLG_CONNECTED;
-      sock->data = server_sock->data;
+      svz_sock_bindings_set (sock, server_sock);
       sock->check_request = server_sock->check_request;
-      sock->idle_func = svz_sock_idle_protect;
+      sock->idle_func = idle_protect;
       sock->idle_counter = 1;
 
       svz_sock_resize_buffers (sock, port->send_buffer_size,
@@ -222,7 +223,7 @@ svz_tcp_accept (svz_socket_t *server_sock)
 
       /* Check access and connect frequency here.  */
       if (svz_sock_check_access (server_sock, sock) < 0 ||
-          svz_sock_check_frequency (server_sock, sock) < 0)
+          check_frequency (server_sock, sock) < 0)
         svz_sock_schedule_for_shutdown (sock);
 
       /*
@@ -244,7 +245,7 @@ svz_tcp_accept (svz_socket_t *server_sock)
  * @code{idle_func} for listening pipe sockets.
  */
 static int
-svz_pipe_accept (svz_socket_t *server_sock)
+pipe_accept (svz_socket_t *server_sock)
 {
 #ifdef __MINGW32__
   DWORD connect;
@@ -399,10 +400,10 @@ svz_pipe_accept (svz_socket_t *server_sock)
   sock->read_socket = svz_pipe_read_socket;
   sock->write_socket = svz_pipe_write_socket;
   svz_sock_setreferrer (sock, server_sock);
-  sock->data = server_sock->data;
+  svz_sock_bindings_set (sock, server_sock);
   sock->check_request = server_sock->check_request;
   sock->disconnected_socket = server_sock->disconnected_socket;
-  sock->idle_func = svz_sock_idle_protect;
+  sock->idle_func = idle_protect;
   sock->idle_counter = 1;
   svz_sock_resize_buffers (sock, port->send_buffer_size,
                            port->recv_buffer_size);
@@ -513,7 +514,7 @@ svz_server_create (svz_portcfg_t *port)
                 svz_log_net_error ("close");
               return NULL;
             }
-          memset (&addr->sin_addr, 0, sizeof (&addr->sin_addr));
+          memset (&addr->sin_addr, 0, sizeof (addr->sin_addr));
         }
 #endif /* SO_BINDTODEVICE */
 
@@ -578,7 +579,7 @@ svz_server_create (svz_portcfg_t *port)
 
   if (port->proto & SVZ_PROTO_PIPE)
     {
-      sock->read_socket = svz_pipe_accept;
+      sock->read_socket = pipe_accept;
       if (svz_pipe_listener (sock,
                              &SVZ_CFG_PIPE (port, recv),
                              &SVZ_CFG_PIPE (port, send)) == -1)
@@ -591,7 +592,7 @@ svz_server_create (svz_portcfg_t *port)
     {
       if (port->proto & SVZ_PROTO_TCP)
         {
-          sock->read_socket = svz_tcp_accept;
+          sock->read_socket = tcp_accept;
         }
       else if (port->proto & SVZ_PROTO_UDP)
         {
